<?php
// +----------------------------------------------------------------------
// | VitPHP
// +----------------------------------------------------------------------
// | 版权所有 2018~2021 藁城区创新网络电子商务中心 [ http://www.vitphp.cn ]
// +----------------------------------------------------------------------
// | VitPHP是一款免费开源软件,您可以访问http://www.vitphp.cn/以获得更多细节。
// +----------------------------------------------------------------------

namespace app\live\controller\mobile;

use app\BaseController;
use think\App;
use think\facade\Db;
use think\facade\Request;

class Base extends BaseController
{
    protected $res = [
        'code' => 0,
        'msg' => 'ok',
        'data' => ''
    ];

    /**
     * Base constructor.
     */
    public function __construct(App $app)
    {
        parent::__construct($app);

        if ($this->isWechat()) {
            $this->cannonFodder();
            $this->getWechatInfo();
            $this->removeCode();
        }
    }

    /**
     * 处理炮灰域名
     * 先判断是否配置了炮灰域名，若配置，再判断当前域名是否为炮灰域名
     * 配置了，且当前不是炮灰域名，就跳转到炮灰域名
     */
    protected function cannonFodder()
    {
        $cannonFodderRange = [];
        if ($cannonFodderRange && !in_array(Request::host(), $cannonFodderRange)) {
            $paohuiDomain = 'b11b23b.05v.cn';
            $thisUrl = Request::scheme() . '://' . $paohuiDomain . Request::url();
            redirect($thisUrl)->send();
        }
    }

    /**
     * 剔除code与state
     */
    protected function removeCode()
    {
        if (!$this->request->isGet() || $this->request->isAjax()) return;
        if (!isset($_GET['code']) && !isset($_GET['state'])) return;
        $userInfo = vit_session('user_wechat');
        if (!$userInfo) return;
        $baseUrl = $this->request->domain() . $this->request->baseUrl();
        $url = $baseUrl;

        unset($_GET['code']);
        unset($_GET['state']);
        if ($_GET) {
            $url = $baseUrl . '?' . http_build_query($_GET);
        }
        \think\facade\Session::save();
        echo "<script>window.location.href='{$url}'</script>";
    }

    protected function emoji2str($str)
    {
        $strEncode = '';
        $length = mb_strlen($str, 'utf-8');
        for ($i = 0; $i < $length; $i++) {
            $_tmpStr = mb_substr($str, $i, 1, 'utf-8');
            if (strlen($_tmpStr) >= 4) {
                $strEncode .= '[[EMOJI:' . rawurlencode($_tmpStr);
            } else {
                $strEncode .= $_tmpStr;
            }
        }
        return $strEncode;
    }

    /* 读数据后重新转换成emoji返回 */
    protected function str2emoji($str)
    {
        $str_arr = explode("[[EMOJI:", $str);
        $new_str = "";
        foreach ($str_arr as $key => $val) {
            $new_str .= rawurldecode($val);
        }
        return $new_str;
    }

    /**
     * 把用户输入的文本转义（主要针对特殊符号和emoji表情）
     */
    public function userTextEncode($str)
    {
        if (!is_string($str)) return $str;
        if (!$str || $str == 'undefined') return '';

        $text = json_encode($str); //暴露出unicode
        $text = preg_replace_callback("/(\\\u[ed][0-9a-f]{3})/i", function ($str) {
            return addslashes($str[0]);
        }, $text); //将emoji的unicode留下，其他不动，这里的正则比原答案增加了d，因为我发现我很多emoji实际上是\ud开头的，反而暂时没发现有\ue开头。
        return json_decode($text);
    }

    /**
     * 解码上面的转义
     */
    public function userTextDecode($str)
    {
        $text = json_encode($str); //暴露出unicode
        $text = preg_replace_callback('/\\\\\\\\/i', function ($str) {
            return '\\';
        }, $text); //将两条斜杠变成一条，其他不动
        return json_decode($text);
    }

    protected function getGuestInfo($wechatInfo)
    {
        $id = input('id');
        $pid = input('pid');
        $uid = input('uid', null);
        $guest = Db::name('live_guest')->where(['pid' => $pid, 'hid' => $id, 'openid' => $wechatInfo['openid']])->find();
//        $guest = Db::name('live_guest')->where(['pid' => $pid, 'hid' => $id, 'openid' => 'oYeHN0fV_2ZifY3ljKT5fzfq9zvk'])->find();
        if (!$guest) {
            Db::name('live_guest')->insert([
                'pid' => $pid,
                'hid' => $id,
                'sid' => $uid,
                'openid' => $wechatInfo['openid'],
                'avatar' => $wechatInfo['headimgurl'],
                'nickname' => $this->userTextEncode($wechatInfo['nickname']),
                'time' => time(),
            ]);
        }
        $guest = Db::name('live_guest')->where(['pid' => $pid, 'hid' => $id, 'openid' => $wechatInfo['openid']])->find();
        $guest['nickname'] = $this->userTextDecode($guest['nickname']);
        return $guest;
    }
    
    /**
     * 获取微信授权信息
     * @return array|mixed|Db|\think\Model|null
     * @throws \WeChat\Exceptions\InvalidResponseException
     * @throws \WeChat\Exceptions\LocalCacheException
     * @throws \think\db\exception\DataNotFoundException
     * @throws \think\db\exception\DbException
     * @throws \think\db\exception\ModelNotFoundException
     */
    protected function getWechatInfo()
    {
        if (!$this->request->isGet() || $this->request->isAjax()) return [];

        $pid = input('pid');
        $wechatInfo = vit_session('user_wechat');
        $thisUrl = Request::scheme() . '://' . Request::host() . Request::url();
        $weixinConfig = Db::name('live_weixin')->where(['pid' => $pid])->find();
        $config = [
            'appsecret' => $weixinConfig['appsecret'],
            'appid' => $weixinConfig['appid'],
        ];
        if (!$wechatInfo || !$wechatInfo['openid']) {

            $wechat = new \WeChat\Oauth($config);
            if (!input('code')) {
                $state = '123';
                $tiaoUrl = $thisUrl;
                if ($weixinConfig['authdomain']) {
                    $tiaoUrl = $weixinConfig['authdomain'];
                    $stateKey = md5(session_id() . $pid . md5(time()) . $thisUrl . time());
                    $state = 'vitphp-' . $stateKey;
                    cache($state, $thisUrl, 60);
                }
                $redirectUrl = $wechat->getOauthRedirect($tiaoUrl, $state, 'snsapi_userinfo');
                redirect($redirectUrl)->send();
            } else {
                $result = $wechat->getOauthAccessToken();
                $wechatInfo = $wechat->getUserInfo($result['access_token'], $result['openid']);
                vit_session('user_wechat', $wechatInfo);
                \think\facade\Session::save();
            }
        }
        return $wechatInfo;
    }

    /**
     * 根据附件，返回可直接查看的url，这个应该放在公共方法里
     * @param $media
     * @return array|string
     */
    protected function getFullMediaUrl($media)
    {
        if (!$media) return '';
        $preg = "/http[s]?:\/\/[\w.]+[\w\/]*[\w.]*\??[\w=&\+\%]*/is";
        $mediaUrl = media($media, 'act');
        if (!preg_match($preg, $mediaUrl)) {
            $mediaUrl = Request::domain() . $mediaUrl;
        }
        return $mediaUrl;
    }

    /**
     * 判断当前是否是微信端
     * @return bool
     */
    protected function isWechat()
    {
        if (strpos($_SERVER['HTTP_USER_AGENT'], 'MicroMessenger') !== false) {
            return true;
        }
        return false;
    }

    /**
     * 判断是否为手机端
     * @return bool
     */
    protected function isMobile()
    {
        // 如果有HTTP_X_WAP_PROFILE则一定是移动设备
        if (isset ($_SERVER['HTTP_X_WAP_PROFILE'])) {
            return TRUE;
        }
        // 如果via信息含有wap则一定是移动设备,部分服务商会屏蔽该信息
        if (isset ($_SERVER['HTTP_VIA'])) {
            if (stristr($_SERVER['HTTP_VIA'], "wap")) return true;
//            return stristr($_SERVER['HTTP_VIA'], "wap") ? TRUE : FALSE;// 找不到为flase,否则为TRUE
        }
        // 判断手机发送的客户端标志,兼容性有待提高
        if (isset ($_SERVER['HTTP_USER_AGENT'])) {
            $clientkeywords = array(
                'mobile',
                'nokia',
                'sony',
                'ericsson',
                'mot',
                'samsung',
                'htc',
                'sgh',
                'lg',
                'sharp',
                'sie-',
                'philips',
                'panasonic',
                'alcatel',
                'lenovo',
                'iphone',
                'ipod',
                'blackberry',
                'meizu',
                'android',
                'netfront',
                'symbian',
                'ucweb',
                'windowsce',
                'palm',
                'operamini',
                'operamobi',
                'openwave',
                'nexusone',
                'cldc',
                'midp',
                'wap'
            );
            // 从HTTP_USER_AGENT中查找手机浏览器的关键字
            if (preg_match("/(" . implode('|', $clientkeywords) . ")/i", strtolower($_SERVER['HTTP_USER_AGENT']))) {
                return TRUE;
            }
        }
        if (isset ($_SERVER['HTTP_ACCEPT'])) { // 协议法，因为有可能不准确，放到最后判断
            // 如果只支持wml并且不支持html那一定是移动设备
            // 如果支持wml和html但是wml在html之前则是移动设备
            if ((strpos($_SERVER['HTTP_ACCEPT'], 'vnd.wap.wml') !== FALSE) && (strpos($_SERVER['HTTP_ACCEPT'], 'text/html') === FALSE || (strpos($_SERVER['HTTP_ACCEPT'], 'vnd.wap.wml') < strpos($_SERVER['HTTP_ACCEPT'], 'text/html')))) {
                return TRUE;
            }
        }
        return FALSE;
    }
}