<?php
// +----------------------------------------------------------------------
// | VitPHP
// +----------------------------------------------------------------------
// | 版权所有 2018~2021 藁城区创新网络电子商务中心 [ http://www.vitphp.cn ]
// +----------------------------------------------------------------------
// | VitPHP是一款免费开源软件,您可以访问http://www.vitphp.cn/以获得更多细节。
// +----------------------------------------------------------------------

namespace app\live\controller\mobile;

use think\facade\Db;
use think\facade\Log;
use think\facade\Request;

class Pay
{
    /**
     * 把用户输入的文本转义（主要针对特殊符号和emoji表情）
     */
    public function userTextEncode($str)
    {
        if (!is_string($str)) return $str;
        if (!$str || $str == 'undefined') return '';

        $text = json_encode($str); //暴露出unicode
        $text = preg_replace_callback("/(\\\u[ed][0-9a-f]{3})/i", function ($str) {
            return addslashes($str[0]);
        }, $text); //将emoji的unicode留下，其他不动，这里的正则比原答案增加了d，因为我发现我很多emoji实际上是\ud开头的，反而暂时没发现有\ue开头。
        return json_decode($text);
    }

    /**
     * 解码上面的转义
     */
    public function userTextDecode($str)
    {
        $text = json_encode($str); //暴露出unicode
        $text = preg_replace_callback('/\\\\\\\\/i', function ($str) {
            return '\\';
        }, $text); //将两条斜杠变成一条，其他不动
        return json_decode($text);
    }

    public function callback()
    {
        $pid = input('pid');
        $weixinConfig = Db::name('live_weixin')->where(['pid' => $pid])->find();
        $config = [
            'appsecret' => $weixinConfig['appsecret'],
            'appid' => $weixinConfig['appid'],
            'mch_id' => $weixinConfig['mchid'],
            'mch_key' => $weixinConfig['apikey']
        ];

        Db::startTrans();
        try {
            $wechat = new \WeChat\Pay($config);
            $result = $wechat->getNotify();
            Log::info('wepay', $result);

            $order = Db::name('live_order')->where(['orderid' => $result['out_trade_no']])->lock(true)->find();
            if($order['ispay']==1){
                $wechat->toXml([
                    'return_code' => 'SUCCESS',
                    'return_msg' => 'OK'
                ]);
                exit();
            }
            Db::name('live_order')->where('id', '=', $order['id'])->update(['ispay' => 1]);

            Db::name('live_guest')->where([
                'pid' => $order['pid'],
                'hid' => $order['rid'],
                'openid' => $order['openid']
            ])->update(['xz_fee_status' => 1]);

            Db::commit();
        } catch (\Exception $e) {
            Db::rollback();

            Log::error('aabb', [$e]);
            Log::error($e->getTraceAsString());
            Log::error($e->getMessage());
            Log::error('aabb', [$e->getMessage()]);
            throw $e;
        }

        $wechat->toXml([
            'return_code' => 'SUCCESS',
            'return_msg' => 'OK'
        ]);
        exit();
    }

    public function place()
    {
        $id = input('id');
        $pid = input('pid');
        $wechatInfo = vit_session('user_wechat');
        $live = Db::name('live_huodong')->where(['pid' => $pid, 'id' => $id])->find();

        $guest = Db::name('live_guest')->where(['pid' => $pid, 'hid' => $id, 'openid' => $wechatInfo['openid']])->find();

        Db::startTrans();
        try {
            list($orderId, $orderNo, $fee) = $this->creatOrder($pid, $id, $guest, $live);
            $options = [
                'body' => $live['title'],
                'out_trade_no' => $orderNo,
                'total_fee' => $fee * 100,
                'openid' => $wechatInfo['openid'],
                'trade_type' => 'JSAPI',
                'notify_url' => Request::domain() . '/live/mobile.pay/callback/pid/'.$pid,
            ];
            $options = $this->placeWechat($pid, $orderId, $options);

            Db::commit();
        } catch (\Exception $e) {
            Db::rollback();
            throw $e;
            return json(['code'=>-1,'msg'=>$e->getMessage()]);
        }

        return json(['code'=>1,'data'=>$options]);
    }

    private function placeWechat($pid, $orderId, $options)
    {
        $weixinConfig = Db::name('live_weixin')->where(['pid' => $pid])->find();
        $config = [
            'appsecret' => $weixinConfig['appsecret'],
            'appid' => $weixinConfig['appid'],
            'mch_id' => $weixinConfig['mchid'],
            'mch_key' => $weixinConfig['apikey']
        ];

        $wechat = new \WeChat\Pay($config);
        $result = $wechat->createOrder($options);
        $options = $wechat->createParamsForJsApi($result['prepay_id']);
        $updateData = [
            'uniontid' => $result['prepay_id']
        ];
        Db::name('live_order')->where('id', '=', $orderId)->update($updateData);

        return $options;
    }

    private function creatOrder($pid, $id, $guest, $live)
    {
        $orderNo = uniqidDate(18);
        $orderData = [
            'pid' => $pid,
            'rid' => $id,
            'orderid' => $orderNo,
            'openid' => $guest['openid'],
            'avatar' => $guest['avatar'],
            'nickname' => $this->userTextEncode($guest['nickname']),
            'ip' => Request::ip(),
            'fee' => $live['xz_pay'],
            'uid' => $guest['id'],
            'createtime' => time(),
            'type' => 1,
            'shopid' => $id,
        ];

        $orderId = Db::name('live_order')->insertGetId($orderData);
        return [$orderId, $orderNo, $live['xz_pay']];
    }

    public function giftCallback()
    {
        $pid = input('pid');
        $weixinConfig = Db::name('live_weixin')->where(['pid' => $pid])->find();
        $config = [
            'appsecret' => $weixinConfig['appsecret'],
            'appid' => $weixinConfig['appid'],
            'mch_id' => $weixinConfig['mchid'],
            'mch_key' => $weixinConfig['apikey']
        ];

        Db::startTrans();
        try {
            $wechat = new \WeChat\Pay($config);
            $result = $wechat->getNotify();
            Log::info('wepay', $result);

            $order = Db::name('live_gift')->where(['orderid' => $result['out_trade_no']])->lock(true)->find();
            $order['nickname'] = $this->userTextDecode($order['nickname']);
            if($order['ispay']==1){
                $wechat->toXml([
                    'return_code' => 'SUCCESS',
                    'return_msg' => 'OK'
                ]);
                exit();
            }
            Db::name('live_gift')->where('id', '=', $order['id'])->update([
                'ispay' => 1,
                'uniontid' => $result['transaction_id'],
                'paytime' => time()
            ]);

            Db::name('live_guest')->where([
                'pid' => $order['pid'],
                'hid' => $order['rid'],
                'openid' => $order['openid']
            ])->update(['xz_fee_status' => 1]);

            $preg = "/http[s]?:\/\/[\w.]+[\w\/]*[\w.]*\??[\w=&\+\%]*/is";
            if (!preg_match($preg, $order['gift_img'])) {
                $order['gift_img'] = Request::domain() . $order['gift_img'];
            }
            $content = "{$order['nickname']}送了{$order['count']}个{$order['gift_name']}<img src=\"{$order['gift_img']}\" class=\"chat_gift_img\">";
            $resData = [
                'pid' => $pid,
                'avatar' => $order['avatar'],
                'nickname' => $this->userTextEncode($order['nickname']),
                'hid' => $order['rid'],
                'content' => $content,
                'time' => time(),
                'openid' => $order['openid'],
                'uid' => $order['uid'] ?? 0,
                'type' => 2,
            ];
            Db::name('live_chat')->insert($resData);

            Db::commit();
        } catch (\Exception $e) {
            Db::rollback();

            $alipayPath = app()->getRootPath() . 'runtime' . DIRECTORY_SEPARATOR . app('http')->getName() . DIRECTORY_SEPARATOR . date('Ym');
            if (!file_exists($alipayPath)) {
                mkdir($alipayPath, 0755, true);
                @chmod($alipayPath, 0755);
            }

            $alipayLogName = date('d') . '.log';
            $alipayLog = $alipayPath . DIRECTORY_SEPARATOR . $alipayLogName;

            $logOld = file_exists($alipayLog) ? file_get_contents($alipayLog) : '';
            $log = $logOld . PHP_EOL . '[' . date('Y-m-d H:i:s') . ']' . 'get:' . $e->getTraceAsString();
            $log = $log . PHP_EOL . '[' . date('Y-m-d H:i:s') . ']' . 'post:' . $e->getMessage();

            file_put_contents($alipayLog, $log);

            Log::error('aabb', [$e]);
            Log::error($e->getTraceAsString());
            Log::error($e->getMessage());
            Log::error('aabb', [$e->getMessage()]);
            throw $e;
        }

        $wechat->toXml([
            'return_code' => 'SUCCESS',
            'return_msg' => 'OK'
        ]);
        exit();
    }

    public function gift()
    {
        $id = input('id');
        $pid = input('pid');
        $count = input('count', 0);
        $index = input('index');
        $wechatInfo = vit_session('user_wechat');
        if (!$wechatInfo) {
            return json(['code' => -1, 'msg' => '请先登录']);
        }

        if(!$count) jsonErrCode('请填写要赠送的数量');

        $live = Db::name('live_huodong')->where(['pid' => $pid, 'id' => $id])->find();
        if(!$live) jsonErrCode('活动不存在');
        if(!$live['gifts']) jsonErrCode('礼物不存在');
        $live['gifts'] = json_decode($live['gifts'], true);
        $gift = [];
        foreach($live['gifts'] as $k =>$item){
            if($k == $index){
                $gift = $item;
                continue;
            }
        }
        if(!$gift) jsonErrCode('礼物不存在！');
        $guest = Db::name('live_guest')->where(['pid' => $pid, 'hid' => $id, 'openid' => $wechatInfo['openid']])->find();
        $guest['nickname'] = $this->userTextDecode($guest['nickname']);
        Db::startTrans();
        try {
            $orderNo = uniqidDate(18);
            $orderData = [
                'pid' => $pid,
                'rid' => $id,
                'orderid' => $orderNo,
                'openid' => $guest['openid'],
                'avatar' => $guest['avatar'],
                'nickname' => $this->userTextEncode($guest['nickname']),
                'ip' => getip(),
                'fee' => $gift['price'] * $count,
                'count' => $count,
                'uid' => $guest['id'],
                'createtime' => time(),
                'type' => 1,
                'gift_index' => $index,
                'gift_name' => $gift['name'],
                'gift_img' => $gift['img'],
                'gift_price' => $gift['price'],
            ];
            $orderId = Db::name('live_gift')->insertGetId($orderData);
            $options = [
                'body' => $gift['name'],
                'out_trade_no' => $orderNo,
                'total_fee' => $orderData['fee'] * 100,
                'openid' => $guest['openid'],
                'trade_type' => 'JSAPI',
                'notify_url' => Request::domain() . '/live/mobile.pay/giftCallback/pid/'.$pid,
            ];
            $options = $this->placeGiftWechat($pid, $orderId, $options);

            Db::commit();
        } catch (\Exception $e) {
            Db::rollback();
            throw $e;
            return json(['code'=>-1,'msg'=>$e->getMessage()]);
        }

        // 处理说话
        $preg = "/http[s]?:\/\/[\w.]+[\w\/]*[\w.]*\??[\w=&\+\%]*/is";
        if (!preg_match($preg, $gift['img'])) {
            $gift['img'] = Request::domain() . $gift['img'];
        }
        $content = "{$guest['nickname']}送了{$count}个{$gift['name']}<img src=\"{$gift['img']}\" class=\"chat_gift_img\">";

        return json([
            'code' => 1,
            'data' => [
                'params' => $options,
                'type' => 2,
                'content' => $content,
                'timestamp' => time(),
                'time' => date('m-d H:i:s')
            ]
        ]);
    }

    private function placeGiftWechat($pid, $orderId, $options)
    {
        $weixinConfig = Db::name('live_weixin')->where(['pid' => $pid])->find();
        $config = [
            'appsecret' => $weixinConfig['appsecret'],
            'appid' => $weixinConfig['appid'],
            'mch_id' => $weixinConfig['mchid'],
            'mch_key' => $weixinConfig['apikey']
        ];

        $wechat = new \WeChat\Pay($config);
        $result = $wechat->createOrder($options);
        $options = $wechat->createParamsForJsApi($result['prepay_id']);
        $updateData = [
            'prepay_id' => $result['prepay_id'],
            'prepay_id_time' => time()
        ];
        Db::name('live_gift')->where('id', '=', $orderId)->update($updateData);

        return $options;
    }
}